// include the library code:
#include <LiquidCrystal.h>
// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 10, 9, 8, 7);

#define encoder0PinA 2
#define encoder0PinB 3
#define encoder0Btn 4

#define NUM_PIEZOS 5

int encoder0Pos = 0;
int valRotary, lastValRotary;

volatile byte systemState, menuState, menuSelectedItem;
// button debounce and control
byte buttonState = HIGH;
byte lastButtonState = HIGH;
boolean buttonPressed = false;
boolean buttonPressedLong = false;
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers
unsigned long longPressDelay = 500;
unsigned long lastLongPressMillis = 0;

// set menu state
void setMenuState(byte newState) {
  switch (newState) {
    case 0:
      lcd.setCursor(0, 1);
      lcd.print("Drum CH:");
      lcd.print(menuSelectedItem + 1);
      // reset all the system menu selctions - to prevent entring into menu - since the button is not processwe when on drum mode
      menuSelectedItem = 0;
      buttonPressed = false;
      buttonPressedLong = false;
      valRotary = lastValRotary;
      lastDebounceTime = millis();
      buttonState = HIGH;
      lastButtonState = HIGH;
      
      break;
    case 1:
      lcd.setCursor(0, 1);
      lcd.print("                ");
      lcd.setCursor(0, 1);
      lcd.print("CH-");
      lcd.print(menuSelectedItem + 1);
      lcd.print(" : ");
      lcd.print(note[menuSelectedItem]);
      break;
    default:
      // do nothing for now
      break;
  }

  //set new value
  menuState = newState;

} //end setMenuState

// check button state
void checkAndSetButtonState() {
  byte reading = digitalRead(encoder0Btn);

  // If the switch changed, due to noise or pressing:
  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  // if the button state and the reading are a match, and its pressed (!reading) and its been long enough
  if (buttonState == reading && !reading && lastDebounceTime + longPressDelay < millis()) {
    if (!buttonPressedLong && lastLongPressMillis + (longPressDelay * 2) < millis()) {
      buttonPressedLong = true;
      lastLongPressMillis = millis();
    } else {
      //Serial.println("HERE setting the last time ");
      lastLongPressMillis = millis();
    } //end if

  } else  if ( lastDebounceTime + debounceDelay < millis() && reading != buttonState) {
    buttonState = reading;
    if (buttonState &&  lastLongPressMillis + 1000 < millis()) {
      buttonPressed = true;
    } //end if
  } //end digitalRead(encoder0Btn);

  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastButtonState = reading;
}

boolean isLongPressButton() {
  boolean returnValue = buttonPressedLong;
  if (buttonPressedLong) {
    buttonPressedLong = false;
    lastLongPressMillis = millis();
  } //end if
  return returnValue;
} // end isLongPressButton


void doEncoder() {
  if (digitalRead(encoder0PinA) == digitalRead(encoder0PinB)) {
    encoder0Pos++;
  } else {
    encoder0Pos--;
  } //end if
  valRotary = encoder0Pos / 2.5;
} //end doEncoder()

void switchToMenu() {
  systemState = 99;
} //end switchToMenu()



// set new system state
void setSystemState(byte newState) {
  switch (newState) {
    case 0:
      detachInterrupt(0);
      attachInterrupt(0, switchToMenu, CHANGE);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("LETS PLAY DRUMS");
      break;
    case 1:
      //attach interupr for the encouder
      detachInterrupt(0);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("MENU");
      setMenuState(0);
      attachInterrupt(0, doEncoder, CHANGE);
      break;
    default:
      // do nothing for now
      break;
  }

  //set new value
  systemState = newState;


} //end setSystemState


void setupGUI() {
  //start lcd
  lcd.begin(16, 2);

  // TO BE REMOVED !!!!
  //set up pins for power
  pinMode(5, OUTPUT);
  digitalWrite(5, HIGH);
  pinMode(6, OUTPUT);
  digitalWrite(6, LOW);

  //set up encoder control pins
  pinMode(encoder0PinA, INPUT_PULLUP);
  pinMode(encoder0PinB, INPUT_PULLUP);
  pinMode(encoder0Btn, INPUT_PULLUP);


  //set new system state
  setSystemState(0);
}


void processMenu() {



  // check and set button state
  checkAndSetButtonState();


  switch (menuState) {
    case 0: {
        // long press will take you back to play mode
        if (isLongPressButton()) {
          setSystemState(0);
        } //end if

        if (buttonPressed) {
          setMenuState(1);
          buttonPressed = false;
          //Serial.println("button pressed on menu");
        } //end if

        // set new menu selction for channels
        byte newMenuSelectedItem = menuSelectedItem;
        if (valRotary > lastValRotary) {
          newMenuSelectedItem = (newMenuSelectedItem + 1) % NUM_PIEZOS ;
        } else if (valRotary < lastValRotary)  {
          if (newMenuSelectedItem == 0) {
            newMenuSelectedItem = NUM_PIEZOS - 1;
          } else {
            newMenuSelectedItem -= 1;
          }
        } //end if

        // if chnage was made lets update the screen
        if (newMenuSelectedItem != menuSelectedItem) {
          // match the value
          menuSelectedItem = newMenuSelectedItem;
          lcd.setCursor(8, 1);
          lcd.print(menuSelectedItem + 1);
        } //end if

        // set last to current so we can track it
        lastValRotary = valRotary;
      } //end case
      break;

    case 1: {
        if (isLongPressButton()) {
          setMenuState(0);
        } //end if

        if (buttonPressed) {
          midiNoteOn(note[menuSelectedItem], 127);
          delay(20);
          //updtae value in EEPROM
          updateAddressValue(NOTE_EEPROM_ADDRESS[menuSelectedItem], note[menuSelectedItem]);
          //turn off
          midiNoteOff(note[menuSelectedItem], 0);
          buttonPressed = false;
        } //end if

        byte newNote = note[menuSelectedItem];

        if (valRotary > lastValRotary) {
          newNote = (newNote + 1) % 70 ;
        } else if (valRotary < lastValRotary)  {
          if (newNote == 0) {
            newNote = 69;
          } else {
            newNote -= 1;
          }
        } //end if

        // if chnage was made lets update the screen
        if (newNote != note[menuSelectedItem]) {
          // match the value
          note[menuSelectedItem] = newNote;
          lcd.setCursor(7, 1);
          lcd.print(newNote);
        } //end if

        // set last to current so we can track it
        lastValRotary = valRotary;

      }//end case



      break;
    default:
      //do nothing for now
      break;
  } //end switch



}












